// RDK.cpp : Definiert die exportierten Funktionen fr die DLL-Anwendung.
//

#include "stdafx.h"
#include ".\RDK.h"
#include "stdio.h"
#include ".\ftcspi.h"
#include <assert.h>

#define RDK_MAX_DEVICE 10
#define RDK_DLLVERSION 0x01000000

static BOOL g_IsInitialized;

static RDK_STATUS InitializeSpiReadStructures(DWORD dwChannel, FTC_INIT_CONDITION* pReadStartCondition, FTC_HIGHER_OUTPUT_PINS *pHighPinsReadActiveStates);
static RDK_STATUS RDK_InitializeDevice(RDK_HANDLE rdkHandle/*, DWORD dwChannel*/);
static BOOL IsRDK(LPCSTR szDeviceName);
static int ExtractAddress(LPCSTR szDeviceName);
static int ExtractOffset(LPCSTR szDeviceName);
static RDK_STATUS GetAddressFromHandle(const RDK_HANDLE handle, LPDWORD lpdwAddress);
static void InitMemory();

typedef struct rdk_deviceinfo {
	int Address;
	DWORD LocId;
	//char SerialNumber[16];
	char Description[64];
	DWORD PressureOffset;
	RDK_HANDLE Handle;

} RDK_DEVICE_INFO,*PRDK_DEVICE_INFO;

RDK_DEVICE_INFO g_DeviceList[RDK_MAX_DEVICE];

// ########################################################################
// Liefert die Anzahl der angeschlossenen RDK-Gerte.
// Auerdem werden Gerteinformationen gespeichert, die spter fr das
// ffnen eines Gerts per Addresse ntig sind; Der Client identifiziert die
// Gerte ber eine im Gert codierte Adresse/Nummer und ffnet das Gert
// ausschlielich ber diese interne Adresse.
// ########################################################################
RDK_API RDK_STATUS WINAPI RDK_GetNumDevices(LPDWORD lpdwNumDevices)
{
	DWORD dwNumSpiDevices=0;
	DWORD dwNumRdkDevices=0;
	DWORD iDevice;
	int iOffset=-1;
	char szDeviceNameBuffer[100];
	DWORD dwLocId;
	int iAddress;

	InitMemory();

	FTC_STATUS ftStatus = SPI_GetNumDevices(&dwNumSpiDevices);

	if (ftStatus != FTC_SUCCESS) return ftStatus;
	
	if (dwNumSpiDevices == 0)
	{
		*lpdwNumDevices = 0;
		return ftStatus;
	}

	for (iDevice=0;iDevice<dwNumSpiDevices;iDevice++)
	{
		ftStatus = SPI_GetDeviceNameLocID(iDevice,szDeviceNameBuffer,100,&dwLocId);

		if (ftStatus==FTC_SUCCESS)
		{
			if (IsRDK(szDeviceNameBuffer))
			{
				iAddress = ExtractAddress(szDeviceNameBuffer);
				iOffset = ExtractOffset(szDeviceNameBuffer);
				if ( iAddress >= 1 && iAddress <= RDK_MAX_DEVICE && iOffset != -1)
				{
					g_DeviceList[iAddress-1].Address = iAddress;
					strcpy(g_DeviceList[iAddress-1].Description,szDeviceNameBuffer);
					g_DeviceList[iAddress-1].LocId = dwLocId;
					g_DeviceList[iAddress-1].PressureOffset = iOffset;

					dwNumRdkDevices++;
				}
			}
		}
	}
	
	*lpdwNumDevices = dwNumRdkDevices;
	return RDK_SUCCESS;
}
//----------------- END RDK_GetNumDevices -----------------

// ########################################################################
// ffnet das RDK-Gert;
//
// Parameter:
// dwDeviceAddress - 1-basierte Gerteadresse, die im Gertenamen der 
//                   RDK-Karte codiert ist.
// prdkHandle      - Zeiger auf Variable vom Wert RDK_HANDLE; die Funktion
//                   schreibt bei Erfolg das zugewiesene Handle in diese Variable.
// ########################################################################
RDK_API RDK_STATUS WINAPI RDK_Open(DWORD dwDeviceAddress, RDK_HANDLE *prdkHandle)
{
	RDK_STATUS status;

	if (dwDeviceAddress > RDK_MAX_DEVICE) return RDK_INVALID_ADDRESS;

	status = SPI_OpenEx(g_DeviceList[dwDeviceAddress-1].Description,g_DeviceList[dwDeviceAddress-1].LocId,prdkHandle);

	if (RDK_SUCCESS == status)
		g_DeviceList[dwDeviceAddress-1].Handle = (*prdkHandle);

	g_IsInitialized = FALSE;

	return status;
}
//----------------- END RDK_Open -----------------


// ########################################################################
// Initialisiert den FTDI-USB-Controller und nimmt Pin-Settings vor.
// ########################################################################
//RDK_API RDK_STATUS WINAPI RDK_InitializeDevice(RDK_HANDLE rdkHandle, DWORD dwChannel)
static RDK_STATUS RDK_InitializeDevice(RDK_HANDLE rdkHandle/*, DWORD dwChannel*/)
{
	static DWORD dwLastChannel=0xffffffff;
	RDK_STATUS Status = RDK_SUCCESS;
	FTC_CHIP_SELECT_PINS   ChipSelectsDisableStates;
	FTC_INPUT_OUTPUT_PINS  HighInputOutputPins;
	//DWORD dwClockDivisor = 29;//Clock divisor 29 entspricht ca 200kHz: 12MHz / ( (1 + Divisor) * 2 );
	DWORD dwClockDivisor = 15;//Clock divisor 29 entspricht ca 200kHz: 12MHz / ( (1 + Divisor) * 2 );


	//if (g_IsInitialized==TRUE && dwLastChannel == dwChannel)
	//	return RDK_SUCCESS;

	if ( g_IsInitialized == FALSE )
		Status = SPI_InitDevice(rdkHandle, dwClockDivisor);
	else
		return RDK_SUCCESS;

	//Status = SPI_InitDevice(rdkHandle, dwClockDivisor);

	//---------------------------------------------
	// Frequenz berprfen
	if ( RDK_SUCCESS == Status )
	{
		DWORD dwFrequency;
		Status = SPI_GetClock(dwClockDivisor,&dwFrequency);
		if ( RDK_SUCCESS != Status ) return Status;
#ifdef _DEBUG
		//if (RDK_SUCCESS == Status)
		//	printf("Aktuelle SPI-Clock-Frequenz: %d Hz\n",dwFrequency);
		//else
		//	printf("Ungueltiger Clock-Divisor\n");
#endif
	}

	if (Status == FTC_SUCCESS)
	{

		// Must set the chip select disable states for all the SPI devices connected to the FT2232C dual device
		// true => slave select disabled; false => slave select enabled
		ChipSelectsDisableStates.bADBUS3ChipSelectPinState = true;//slave disabled
		ChipSelectsDisableStates.bADBUS4GPIOL1PinState = true;    //slave disabled
		ChipSelectsDisableStates.bADBUS5GPIOL2PinState = true;    //slave disabled
		ChipSelectsDisableStates.bADBUS6GPIOL3PinState = true;    //slave disabled
		ChipSelectsDisableStates.bADBUS7GPIOL4PinState = true;    //slave disabled

		//bPinXInputOutputState: true => Output; false => Input
		//bPinXLowHighState    : true => high  ; false => low
		HighInputOutputPins.bPin1InputOutputState = true;//ACBus0_GPIOH1 ?
		HighInputOutputPins.bPin1LowHighState = true;    //ACBus0_GPIOH1 ?
		HighInputOutputPins.bPin2InputOutputState = true;//ACBus1_GPIOH2 ?
		HighInputOutputPins.bPin2LowHighState = true;    //ACBus1_GPIOH2 ?
		HighInputOutputPins.bPin3InputOutputState = true;//ACBus2_GPIOH3 ?
		HighInputOutputPins.bPin3LowHighState = true;    //ACBus2_GPIOH3 ?
		HighInputOutputPins.bPin4InputOutputState = true;//ACBus3_GPIOH4 ?
		HighInputOutputPins.bPin4LowHighState = true;    //ACBus3_GPIOH4 ?

		Status = SPI_SetGPIOs(rdkHandle, &ChipSelectsDisableStates, &HighInputOutputPins);
	}

	if (RDK_SUCCESS==Status)
	{
		g_IsInitialized = TRUE;
		//dwLastChannel = dwChannel;
	}
	return Status;
}
//----------------- END RDK_InitializeDevice -----------------

// ########################################################################
// Hole Druck- und Temperaturwerte fr alle Kanle;
// Diese Funktion ist performanter als die Funktion fr den Abruf von Werten einzelner
// Kanle, da hier die Sensorzyklen besser ausgenutzt werden knnen  (STIMMT NICHT!!!!
// Da viele einzelne USB-Befehle erfolgen ist diese Funktion um ein Vielfaches langsamer!!!!
// Die Werte fr die
// Mittelwertbildung werden jeweils reihum von den angeschlossenen Sensoren geholt.
// Dadurch muss nicht unttig gewartet werden, bis der aktuelle Zyklus durchlaufen ist und
// der Sensor neue Werte ermittelt hat.
// Der zurckgelieferte Wert ist ein Mittelwert ber 100 Einzelwerte. Der Wertebereich
// ist [0..4095]. Der Druckwert ist nicht in bar umgerechnet, sondern der um den Offset
// bereinigte und auf 12 Bit verkrzte Mittelwert. Zur Umrechnung in Bar siehe Sensordatenblatt.
//
// Parameter:
// rdkHandle     
// dwChannel    - 0-basierter Index des auszulesenden Sensors
// lpdwDruck    - Zeiger auf eine Variable vom Typ DWORD, in die der ausgelesene
//                Druckwert geschrieben wird.
// ########################################################################
RDK_API RDK_STATUS WINAPI RDK_GetDruckAll(RDK_HANDLE rdkHandle, PressureDataDwordBuffer pressuredata, TemperatureDataDwordBuffer temperaturedata)
{
	RDK_STATUS Status;

	FTC_INIT_CONDITION     ReadStartCondition;
	FTC_HIGHER_OUTPUT_PINS HighPinsReadActiveStates;
	
	WriteControlByteBuffer WriteControlBuffer;
	ReadDataByteBuffer     ReadDataBuffer;

	DWORD dwValuesPerArithmeticMean = 100; //Anzahl der Werte, ber die gemittelt werden soll.
	DWORD dwNumDataBytesToRead      = 4;	//2 Bytes Druck, 2 Bytes Temperatur
	DWORD dwNumDataBitsToRead       = 8 * dwNumDataBytesToRead;
	DWORD dwNumDataBytesReturned;

	DWORD arr_dwPressureSum[RDK_MAX_CHANNEL];
	DWORD arr_dwTemperatureSum[RDK_MAX_CHANNEL];
	DWORD dwPressure;
	DWORD dwTemperature;

	if ( g_IsInitialized != TRUE)
	{
		Status = RDK_InitializeDevice(rdkHandle);
		if ( RDK_SUCCESS != Status ) return Status;
	}
	
	ZeroMemory(arr_dwPressureSum,sizeof(DWORD)*RDK_MAX_CHANNEL);
	ZeroMemory(arr_dwTemperatureSum,sizeof(DWORD)*RDK_MAX_CHANNEL);

	WriteControlBuffer[0]=255;
	WriteControlBuffer[1]=255;
	// Read and aggregate values
	for ( DWORD dwCycle=0; dwCycle<dwValuesPerArithmeticMean; dwCycle++ )
	{
		for ( DWORD dwChannel=0; dwChannel<RDK_MAX_CHANNEL; dwChannel++ )
		{
			Status = InitializeSpiReadStructures(dwChannel,&ReadStartCondition,&HighPinsReadActiveStates);
			if ( RDK_SUCCESS != Status ) return Status;

			Status = SPI_Read( rdkHandle, &ReadStartCondition,
				               TRUE, FALSE, 8, &WriteControlBuffer, 1,
				               TRUE, FALSE, dwNumDataBitsToRead, &ReadDataBuffer, &dwNumDataBytesReturned,
							   &HighPinsReadActiveStates );

			if ( RDK_SUCCESS != Status ) return Status;
			if ( dwNumDataBytesReturned != dwNumDataBytesToRead ) return RDK_DATA_BYTES_RETURNED;

			dwPressure = (ReadDataBuffer[0]<<8) + ReadDataBuffer[1];
			arr_dwPressureSum[dwChannel] += dwPressure;
			dwTemperature = (ReadDataBuffer[2] << 8) + ReadDataBuffer[3];
			arr_dwTemperatureSum[dwChannel] += dwTemperature;
		}
	}

	// Calculate arithmetic mean
	for ( DWORD dwChannel=0; dwChannel<RDK_MAX_CHANNEL; dwChannel++ )
	{
		pressuredata[dwChannel]    = (DWORD)(arr_dwPressureSum[dwChannel]    / (double)dwValuesPerArithmeticMean);
		temperaturedata[dwChannel] = (DWORD)(arr_dwTemperatureSum[dwChannel] / (double)dwValuesPerArithmeticMean);
	}

	return RDK_SUCCESS;
}


// ########################################################################
// Hole Druck- und Temperaturwert fr einen Kanal;
// Der gewnschte Sensor ist durch Device-Handle und Kanal eindeutig bestimmt.
// Der zurckgelieferte Wert ist ein Mittelwert ber 100 Einzelwerte. Der Wertebereich
// ist [0..4095]. Der Druckwert ist nicht in bar umgerechnet, sondern der um den Offset
// bereinigte und auf 12 Bit verkrzte Mittelwert. Zur Umrechnung in Bar siehe Sensordatenblatt.
//
// Parameter:
// rdkHandle     
// dwChannel      - 0-basierter Index des auszulesenden Sensors
// lpdwDruck      - Zeiger auf eine Variable vom Typ DWORD, in die der ausgelesene
//                  Druckwert geschrieben wird.
// lpdwTemperatur - Zeiger auf eine Variable vom Typ DWORD, in die der ausgelesene
//                  Temperaturwert geschrieben wird.
// ########################################################################
RDK_API RDK_STATUS WINAPI RDK_GetDruck(RDK_HANDLE rdkHandle, DWORD dwChannel, LPDWORD lpdwDruck, LPDWORD lpdwTemperatur, BOOL bRawDataScreenDump)
{
	RDK_STATUS Status;

	FTC_INIT_CONDITION     ReadStartCondition;
	FTC_HIGHER_OUTPUT_PINS HighPinsReadActiveStates;
	
	WriteControlByteBuffer WriteControlBuffer;
	ReadDataByteBuffer     ReadDataBuffer;

	//ACHTUNG: Die Parameter bClockOutControBitsPosEdge und bClockInDataBitsPosEdge und ReadStartCondition.bClockPinState 
	//         mssen aufeinander abgestimmt sein. Sollen Daten also mit der positiven (steigenden) Flanke bernommen werden,
	//         so muss ReadStartCondition.bClockPinState ebenfalls den Wert true haben.
	BOOL bClockOutControBitsMSBFirst = true;
	BOOL bClockOutControBitsPosEdge  = false;
	BOOL bClockInDataBitsMSBFirst    = true;
	BOOL bClockInDataBitsPosEdge     = false;
	
	DWORD dwNumControlBitsToWrite  = 8;
	DWORD dwNumControlBytesToWrite = 1;

	//Der Sensor braucht laut Datenblatt 250us fr einen Durchlauf; werden die Daten schneller gelesen, so werden (zumindest fr
	//die Druckdaten) innerhalb des Zyklus immer die selben Werte zurckgeliefert. Da das Programm eine Mittlung ber jeweils
	//100 Werte vorsieht, mssen also Werte aus 100 Sensordurchlufen gesammelt werden. Aus jedem Sensordurchlauf wird jeweils
	//nur der erste Wert in die Mittelwertbildung einbezogen.
	int   dwValuesPerArithmeticMean = 100; //Anzahl der Werte, ber die gemittelt werden soll.
	int   dwBytesPerSensorCycle     = 8;   //Anzahl der Bytes, die ungefhr pro Sensordurchlauf bei 500kHz gelesen werden knnen.
	DWORD dwNumDataBytesToRead     = (DWORD)(dwValuesPerArithmeticMean * dwBytesPerSensorCycle);
	DWORD dwNumDataBitsToRead      = 8 * dwNumDataBytesToRead;
	DWORD dwNumDataBytesReturned;

	//if ( g_IsInitialized != TRUE)
	//{
		Status = RDK_InitializeDevice(rdkHandle/*,dwChannel*/);
		if ( RDK_SUCCESS != Status ) return Status;
	//}

	Status = InitializeSpiReadStructures(dwChannel,&ReadStartCondition,&HighPinsReadActiveStates);
	if ( RDK_SUCCESS != Status ) return Status;

	WriteControlBuffer[0]=255;
	WriteControlBuffer[1]=255;

	//////////////////////////////////////////////////////////////////////////////////////////////////////////
	//<TEST_CODE>
	if (bRawDataScreenDump==TRUE)
	{
		DWORD dwMinVal=0xffffffff;
		DWORD dwMaxVal=0;
		dwNumDataBytesToRead = 16 /*800*/;
		dwNumDataBitsToRead  = 8 * dwNumDataBytesToRead;
		for (int i=0; i<50; i++)
		{
			Status = SPI_Read(rdkHandle, &ReadStartCondition, bClockOutControBitsMSBFirst, bClockOutControBitsPosEdge,
							dwNumControlBitsToWrite, &WriteControlBuffer, dwNumControlBytesToWrite,
							bClockInDataBitsMSBFirst, bClockInDataBitsPosEdge, dwNumDataBytesToRead*8/*524280*/,
							&ReadDataBuffer, &dwNumDataBytesReturned, &HighPinsReadActiveStates);

			assert(dwNumDataBytesReturned == dwNumDataBytesToRead);

			if ( RDK_SUCCESS != Status )
			{
				//printf("Error code: %d\n",Status);
				return Status;
			}
			//assert(dwNumDataBytesReturned == dwNumDataBytesToRead);
			//printf("Zyklus %4d\n",i);
			for (DWORD j=0; j<dwNumDataBytesToRead; j+=4/*dwNumDataBytesToRead*/)
			{
				if (ReadDataBuffer[j] < dwMinVal) dwMinVal = ReadDataBuffer[j];
				if (ReadDataBuffer[j] > dwMaxVal) dwMaxVal = ReadDataBuffer[j];
				for (int k=0; k<4; k++)
				{
					//printf("%#.2x    ",ReadDataBuffer[j+k]);//Hex print
					printf("%.3d    ",ReadDataBuffer[j+k]);//Dec print
				}
				printf("\n");
			}
			Sleep(50);
			printf("\n");
		}

		printf("Minimum value of first byte: %3d\n",dwMinVal);
		printf("Maximum value of first byte: %3d\n",dwMaxVal);
		return RDK_SUCCESS;
	}
	//</TEST_CODE>
	//////////////////////////////////////////////////////////////////////////////////////////////////////////

	Status = SPI_Read(rdkHandle, &ReadStartCondition, bClockOutControBitsMSBFirst, bClockOutControBitsPosEdge,
					dwNumControlBitsToWrite, &WriteControlBuffer, dwNumControlBytesToWrite,
					bClockInDataBitsMSBFirst, bClockInDataBitsPosEdge, dwNumDataBytesToRead*8/*524280*/,
					&ReadDataBuffer, &dwNumDataBytesReturned, &HighPinsReadActiveStates);

	assert(dwNumDataBytesReturned == dwNumDataBytesToRead);

	int iCycle = 0;
	DWORD sumPressure=0;
	DWORD sumTemperature=0;
	DWORD dwTemperature=0;
	DWORD dwPressure=0;
	for (DWORD i=0; i<dwNumDataBytesToRead; i+=dwBytesPerSensorCycle)
	{
		//Die Druck- und Temperaturwerte werden jeweils als 15-Bit-Werte in zwei Bytes geschickt,
		//wobei zuerst das Byte mit dem MSB kommt. Ausgewertet werden also Bit 0-14,  Bit 15 (MSB)
		//wird verworfen.

		//sumPressure    += ((ReadDataBuffer[i]   << 8) & 0xff00);//Set MSB to zero
		//sumPressure    +=   ReadDataBuffer[i+1];
		dwPressure = (ReadDataBuffer[i]<<8) + ReadDataBuffer[i+1];
		sumPressure += dwPressure;
		//sumTemperature += ((ReadDataBuffer[i+2] << 8) & 0xff00);//Set MSB to zero
		//sumTemperature +=   ReadDataBuffer[i+3];
		dwTemperature = (ReadDataBuffer[i+2] << 8) + ReadDataBuffer[i+3];
		sumTemperature += dwTemperature;
		iCycle++;
	}

	//Der Sensor (Sensortechnics HCE-Serie) arbeitet intern mit 14 Bit arbeitet, liefert aber einen
	//15-Bit-Wert; da wir nur einen Wertebereich von 12 Bit bentigen, verwerfen wir nach der
	//Mittelwertbildung die drei niedrigwertigen Bits.
	assert(iCycle == dwValuesPerArithmeticMean);
	
	//DWORD dwAddress;
	//DWORD dwPressureOffset;
	DWORD meanPressure=0;
	//DWORD meanPressureNoOffset=0;
	DWORD meanTemperature=0;

	//GetAddressFromHandle(rdkHandle,&dwAddress);
	//dwPressureOffset = g_DeviceList[dwAddress-1].PressureOffset;

	meanPressure = (DWORD)(sumPressure  / (double)iCycle);//remove 3 LSBs

	//meanPressureNoOffset = ((DWORD)(sumPressure  / (double)iCycle)) >> 3;
	//meanPressureNoOffset &= 0x0fff;

	//meanPressure = ((DWORD)((sumPressure - dwPressureOffset*100)  / (double)iCycle)) >> 3 /*& 0x0fff*/;
	//meanPressure &= 0x0fff;

	meanTemperature = ((DWORD)(sumTemperature / (double)iCycle));

	//meanTemperature = ((DWORD)(sumTemperature / (double)iCycle)) >> 3 /*& 0x0fff*/;
	//meanTemperature &= 0x0fff;

	*lpdwDruck = meanPressure;
	*lpdwTemperatur = meanTemperature;

#ifdef _DEBUG
	//printf("%d\t%d\t%d\n",meanPressureNoOffset,meanPressure,meanTemperature);
#endif

	return Status;
}
//----------------- END RDK_GetDruck -----------------

// ########################################################################
// Hole Temperaturwert;
// Der gewnschte Sensor ist durch Device-Handle und Kanal eindeutig bestimmt.
// ########################################################################
RDK_API RDK_STATUS WINAPI RDK_GetTemp(RDK_HANDLE rdkHandle, DWORD dwChannel, LPDWORD lpdwTemp)
{
	return RDK_NOT_IMPLEMENTED;
}
//----------------- END RDK_GetTemp -----------------

// ########################################################################
// Schliet das RDK-Gert
// ########################################################################
RDK_API RDK_STATUS WINAPI RDK_Close(RDK_HANDLE rdkHandle)
{
	RDK_STATUS status;
	DWORD dwAddress=0;

	status = SPI_Close(rdkHandle);
	
	if (RDK_SUCCESS == status)
	{
		GetAddressFromHandle(rdkHandle,&dwAddress);
		g_DeviceList[dwAddress-1].Handle = NULL;
	}

	return status;
}
//----------------- END RDK_Close -----------------


// ########################################################################
// Liefert einen Versionsstring;
//
// Format: Hauptversion.Nebenversion.Revision.Build 
// ########################################################################
RDK_API RDK_STATUS WINAPI RDK_GetDllVersion(LPSTR lpDllVersionBuffer, DWORD dwBufferSize, LPDWORD lpdwVersion)
{
	//TODO: Versionsstring auf Basis der internen Version (RDK_DLLVERSION) generieren
	if (dwBufferSize < 4) return RDK_INSUFFICIENT_RESOURCES;
	
	strcpy(lpDllVersionBuffer,"1.0");
	*lpdwVersion = RDK_DLLVERSION;

	return RDK_SUCCESS;
}
//----------------- END RDK_GetDllVersion -----------------


// ########################################################################
// Liefert einen Fehlerbeschreibungstext.
// Die Puffergre dwBufferSize sollte mindestens 100 betragen.
// ########################################################################
RDK_API RDK_STATUS WINAPI RDK_GetErrorCodeString(RDK_STATUS StatusCode, LPSTR lpErrorMessageBuffer, DWORD dwBufferSize)
{
	FTC_STATUS status;
	if ( StatusCode < RDK_ERRORCODE_BASE )
	{
		status = SPI_GetErrorCodeString("EN",StatusCode,lpErrorMessageBuffer,dwBufferSize);
		return status;
	}

	if (dwBufferSize < 100) return RDK_INSUFFICIENT_RESOURCES;

	//TODO: Texte fr RDK-spezifische Fehlerzustnde definieren und zurckgeben.
	switch(StatusCode)
	{
	case RDK_NOT_IMPLEMENTED:
		strcpy(lpErrorMessageBuffer,"Aufgerufene Funktion ist nicht implementiert.");
		break;
	case RDK_INVALID_ADDRESS:
		strcpy(lpErrorMessageBuffer,"Geraeteadresse ausserhalb des erlaubten Wertebereichs [1..10].");
		break;
	case RDK_INVALID_CHANNEL:
		strcpy(lpErrorMessageBuffer,"Sensornummer/Kanal ungueltig.");
		break;
	case RDK_DEVICE_NOT_INITIALIZED:
		strcpy(lpErrorMessageBuffer,"USB-Chip nicht initialisiert. Vor Leseoperation bitte RDK_InitializeDevice() aufrufen.");
		break;
	default:
		strcpy(lpErrorMessageBuffer,"Unbekannter Fehler oder kein Fehlertext definiert.");
		break;
	}

	return RDK_SUCCESS;
}
//----------------- END RDK_GetErrorCodeString -----------------


// ########################################################################
// Initialisiert die Datenstrukturen fr den Aufruf von SPI_Read(...)
// ########################################################################
static RDK_STATUS InitializeSpiReadStructures(DWORD dwChannel, FTC_INIT_CONDITION* pReadStartCondition, FTC_HIGHER_OUTPUT_PINS *pHighPinsReadActiveStates)
{
	pReadStartCondition->bClockPinState = false;
	pReadStartCondition->bDataOutPinState = true;
	pReadStartCondition->bChipSelectPinState = true;//disable chip select

	//Slave select fr den gewhlten Kanal einstellen
	switch (dwChannel)
	{
	case 0:
		pReadStartCondition->dwChipSelectPin = ADBUS3ChipSelect;
		pReadStartCondition->bChipSelectPinState = true;
		break;
	case 1:
		pReadStartCondition->dwChipSelectPin = ADBUS4GPIOL1;
		pReadStartCondition->bChipSelectPinState = true;
		break;
	case 2:
		pReadStartCondition->dwChipSelectPin = ADBUS5GPIOL2;
		pReadStartCondition->bChipSelectPinState = true;
		break;
	case 3:
		pReadStartCondition->dwChipSelectPin = ADBUS6GPIOL3;
		pReadStartCondition->bChipSelectPinState = true;
		break;
	//case 4:
	//	ReadStartCondition.dwChipSelectPin = ADBUS7GPIOL4;
	//	ReadStartCondition.bChipSelectPinState = true;
	//	break;
	default:
		return RDK_INVALID_CHANNEL;
	}

	// Configuration of GPIOH1-4 (ACBUS0-3) pins for read operation
	//bPinXState: true => use pin; false => don't use pin
	//bPinXActiveState: true => set pin to high during read; false => set pin to low during read
	pHighPinsReadActiveStates->bPin1ActiveState = false;
	pHighPinsReadActiveStates->bPin1State = false;       
	pHighPinsReadActiveStates->bPin2ActiveState = false;
	pHighPinsReadActiveStates->bPin2State = false;       
	pHighPinsReadActiveStates->bPin3ActiveState = false;
	pHighPinsReadActiveStates->bPin3State = false;       
	pHighPinsReadActiveStates->bPin4ActiveState = false;
	pHighPinsReadActiveStates->bPin4State = false;       

	return RDK_SUCCESS;
}

// ########################################################################
// Prft, ob der bergebene SPI-Device-Name eine RDK bezeichnet.
// ########################################################################
static BOOL IsRDK(LPCSTR lpszDeviceName)
{
	//Namensschema: "RDK1.0_1_3277_... [A|B]"
	//if (strcmp(description,"RDK1.0_1_3277 A") != 0 ) return FALSE;

	if ( NULL == lpszDeviceName ) return FALSE;
	if ( '\0' == lpszDeviceName[0] ) return FALSE;
	if ( lpszDeviceName[0] != 'R' ) return FALSE;
	if ( lpszDeviceName[1] != 'D' ) return FALSE;
	if ( lpszDeviceName[2] != 'K' ) return FALSE;
	if ( lpszDeviceName[3] != '1' ) return FALSE;
	if ( lpszDeviceName[4] != '.' ) return FALSE;
	if ( lpszDeviceName[5] != '0' ) return FALSE;
	if ( lpszDeviceName[6] != '_' ) return FALSE;

	return TRUE;
}
//----------------- END IsRdkDevice -----------------

// ########################################################################
// Extrahiert die Gerteadresse aus dem Gertenamen. Die Gerteadresse ist
// eine ganze Zahl aus dem Intervall [1..MAX_DEVICE]. Der Clientcode benutzt
// diese Nummer, um ein Handle auf die Druckkarte zu erhalten. Diese 
// Adressierung dient der festen Zuordnung von  Messwerte zu Messpunkten.
//
// Der Gertename ist nach folgendem Schema aufgebaut:
//
// NameVersion_Adresse_NullwertSensor1[weitere Nullwerte] A
// (Das A steht fr den Kanal des USB-Chips. FT2232-USB-Controller untersttzen
// SPI nur auf Kanal A).
//
// PRECONDITIONS
// - szDeviceName ist ein gltiger RDK-Name => IsRDK(LPCSTR szDeviceName)=TRUE;
// ########################################################################
// TODO: Endgltiges Namensschema festlegen und Code anpassen
static int ExtractAddress(LPCSTR szDeviceName)
{
	int addr=-1;
	
	if (szDeviceName[7]<'0' || szDeviceName[7]>'9')
		return -1;//Invalid address

	addr = szDeviceName[7]-'0';//1-digit address

	if (szDeviceName[8]>='0' && szDeviceName[8]<='9')
		addr = 10*addr + szDeviceName[8];//2-digit address

	return addr;
}
//----------------- END ExtractAddress -----------------

// ########################################################################
// Die in den FTDI-USB-Chip einprogrammierte Bezeichnung enthlt auch den 
// Offset-Wert des Drucksensors, d.h. den niedrigsten Count, den der Sensor
// zurckliefern kann (ACHTUNG! Dieser Wert muss erst noch umgerechnet werden,
// um zu einem tatschlichen Druckwert in bar zu kommen).
// Dieser Wert wird spter von den gelieferten Werten abgezogen, um so einen
// Wertbereich von [0..4095] zu erhalten.
//
// Return: Der Offset-Wert. Wenn der Gertename nicht erfolgreich ausgewertet
//         werden kann, liefert die Funktion -1 zurck.
// ########################################################################
static int ExtractOffset(LPCSTR szDeviceName)
{
	int offset=0;
	BOOL scan = TRUE;
	int pos = 9;

	while (scan)
	{
		if (szDeviceName[pos]>='0' && szDeviceName[pos]<='9')
		{
			offset = offset * 10 + (szDeviceName[pos]-'0');
			pos++;
		}
		else
		{
			scan=FALSE;
		}
	}

	if (9==pos) return -1; //invalid offset

	return offset;
}
//----------------- END ExtractOffset -----------------

// ########################################################################
// Ermittle die zum Handle passende Gertenummer
// ########################################################################
static RDK_STATUS GetAddressFromHandle(const RDK_HANDLE handle, LPDWORD lpdwAddress)
{
	int index=0;
	BOOL search=TRUE;

	while (search)
	{
		if (handle == g_DeviceList[index++].Handle) search = FALSE;
		if (RDK_MAX_DEVICE == index) search = FALSE;
	}
	
	if (RDK_MAX_DEVICE == index)
	{
		return RDK_INVALID_HANDLE;
	}
	else
	{
		*lpdwAddress = index;
		return RDK_SUCCESS;
	}
}
//----------------- END GetAddressFromHandle -----------------


// ########################################################################
// Setzt den Speicher fr die Gerteliste auf 0, um unerwnschte Effekte
// zu verhindern (an Stellen, wo ggfs. keine berprfung der Werte vorgenommen
// wird).
// ########################################################################
static void InitMemory()
{
	ZeroMemory(g_DeviceList,sizeof(RDK_DEVICE_INFO)*RDK_MAX_DEVICE);
}
//----------------- END InitMemory -----------------
