/*++

Copyright (c) 2009 Romano Przisionstechnik GmbH

Module Name:

    RDK.h

Abstract:

    API DLL fr eine Drucksensorkarte mit FTDI FT2232D-USB-Chip mit SPI-Schnittstelle
	zwischen USB-Controller und Drucksensoren

	RDK library definitions

    2009-12-01    jw     Created.
	
--*/


#ifndef RDK_H
#define RDK_H


// The following ifdef block is the standard way of creating macros
// which make exporting from a DLL simpler.  All files within this DLL
// are compiled with the RDK_EXPORTS symbol defined on the command line.
// This symbol should not be defined on any project that uses this DLL.
// This way any other project whose source files include this file see
// RDK_API functions as being imported from a DLL, whereas this DLL
// sees symbols defined with this macro as being exported.

#ifdef RDK_EXPORTS
#define RDK_API __declspec(dllexport)
#else
#define RDK_API __declspec(dllimport)
#endif

typedef DWORD RDK_HANDLE;
typedef ULONG RDK_STATUS;
#define RDK_MAX_CHANNEL 4

// Renamed FTC error codes
#define RDK_SUCCESS 0 // FTC_SUCCESS, FTC_OK
#define RDK_INVALID_HANDLE 1 // FTC_INVALID_HANDLE
#define RDK_DEVICE_NOT_FOUND 2 // FTC_DEVICE_NOT_FOUND
#define RDK_DEVICE_NOT_OPENED 3 // FTC_DEVICE_NOT_OPENED
#define RDK_IO_ERROR 4 // FTC_IO_ERROR
#define RDK_INSUFFICIENT_RESOURCES 5 // FTC_INSUFFICIENT_RESOURCES

// Native RDK error code range
#define RDK_ERRORCODE_BASE  100
#define RDK_ERRORCODE_HIGH  102
// Native RDK error codes
#define RDK_NOT_IMPLEMENTED 100
#define RDK_INVALID_ADDRESS 101
#define RDK_INVALID_CHANNEL 102
#define RDK_DEVICE_NOT_INITIALIZED 103
#define RDK_DATA_BYTES_RETURNED 104

#ifdef __cplusplus
extern "C" {
#endif

typedef DWORD PressureDataDwordBuffer[RDK_MAX_CHANNEL];
typedef PressureDataDwordBuffer* PPressureDataDwordBuffer;
typedef DWORD TemperatureDataDwordBuffer[RDK_MAX_CHANNEL];
typedef TemperatureDataDwordBuffer* PTemperatureDataDwordBuffer;

RDK_API RDK_STATUS WINAPI RDK_GetNumDevices(LPDWORD lpdwNumDevices);

RDK_API RDK_STATUS WINAPI RDK_Open(DWORD dwDeviceAddress, RDK_HANDLE *pftHandle);

//RDK_API RDK_STATUS WINAPI RDK_InitializeDevice(RDK_HANDLE rdkHandle, DWORD dwChannel);

RDK_API RDK_STATUS WINAPI RDK_GetDruck(RDK_HANDLE rdkHandle, DWORD dwChannel, LPDWORD lpdwDruck, LPDWORD lpdwTemperatur, BOOL bRawDataScreenDump=FALSE);

RDK_API RDK_STATUS WINAPI RDK_GetDruckAll(RDK_HANDLE rdkHandle, PressureDataDwordBuffer pressuredata, TemperatureDataDwordBuffer temperaturedata);

//RDK_API RDK_STATUS WINAPI RDK_GetTemp(RDK_HANDLE rdkHandle, DWORD dwChannel, LPDWORD lpdwTemp);

RDK_API RDK_STATUS WINAPI RDK_Close(RDK_HANDLE rdkHandle);

RDK_API RDK_STATUS WINAPI RDK_GetDllVersion(LPSTR lpDllVersionBuffer, DWORD dwBufferSize);

RDK_API RDK_STATUS WINAPI RDK_GetErrorCodeString(RDK_STATUS StatusCode, LPSTR lpErrorMessageBuffer, DWORD dwBufferSize);

#ifdef __cplusplus
}
#endif


#endif  /* RDK_H */